import { useEffect } from "react";
import { useState } from "react";
import { Flight } from "../Types";
import { fetchZones } from "../Api";
import { Stores, addData, deleteData, getStoreData, updateData, initDB } from '../db';

export const useFetchDestination = (destination: string | undefined) => {
    const [error, setError] = useState<string | null>(null);
    const [zones, setZones] = useState<Flight[]>([]);

    const isToday = (someDate: Date) => {
        let today = new Date();
        return someDate.getDate() == today.getDate() &&
            someDate.getMonth() == today.getMonth() &&
            someDate.getFullYear() == today.getFullYear()
    }

    useEffect(() => {
        const intervalId = setInterval(() => {
            let today = localStorage.getItem('date')

            if (today && !isToday(new Date(today))) {
                getStoreData<Flight>(Stores.Arrival)
                    .then((data) => {
                        if (data) {
                            data.map((u) => {
                                deleteData(Stores.Arrival, u.id)
                            })
                        }
                    })
                localStorage.setItem('date', new Date().toString())
            }

        }, 36000)

        return () => clearInterval(intervalId);
    }, [])


    useEffect(() => {
        setError(null);
        let newUpdate = new Date().toISOString()

        localStorage.setItem('date', new Date().toString())

        initDB().then((x) => {
            console.log(x)
            getStoreData<Flight>(Stores.Arrival)
                .then((data) => {
                    console.log(data)
                    if (data && data.length > 0) {
                        data.sort((a, b) => new Date(a.departure_time).getTime() - new Date(b.departure_time).getTime())
                        setZones(data)
                    } else {
                        fetchZones(undefined, destination, null)
                            .then((data) => {
                                localStorage.setItem('lastUpdated', newUpdate)
                                let toAdd: Flight[] = []
                                data.map((u) => {
                                    if (u.status != 'Deleted') {
                                        addData(Stores.Arrival, u)
                                        toAdd.push(u)
                                    }
                                })
                                toAdd.sort((a, b) => new Date(a.departure_time).getTime() - new Date(b.departure_time).getTime())
                                setZones(toAdd);
                            })
                            .catch((error) => { });
                    }
                })
        }
        )

    }, [origin]);

    useEffect(() => {
        const intervalId = setInterval(() => {
            let lastUpdate = localStorage.getItem('lastUpdated')
            let newUpdate = new Date().toISOString()

            fetchZones(undefined, destination, lastUpdate)
                .then((data) => {
                    localStorage.setItem('lastUpdated', newUpdate)
                    let toAdd: Flight[] = []
                    let toRemove: Flight[] = []

                    zones.forEach((c, i) => {
                        let index = data.findIndex(x => x.id === c.id)
                        if (index >= 0) {
                            console.log(data[index].departure_time + 'nuevo')
                            if (data[index].status == 'Deleted' || new Date(data[index].departure_time) < new Date()) {
                                console.log("sacamos")
                                toRemove.push(data[index])
                                deleteData(Stores.Arrival, c.id)
                            } else {
                                toAdd.push(data[index]);
                                updateData(Stores.Arrival, c.id, data[index])
                            }
                        } else {
                            console.log(new Date(c.departure_time))
                            if (c.status == 'Deleted' || new Date(c.departure_time) < new Date()) {
                                console.log("sacamos?")
                                toRemove.push(c);
                                deleteData(Stores.Arrival, c.id)
                            } else {
                                toAdd.push(c);
                            }
                        }
                    });

                    let filtered = data.filter(o =>
                        !toAdd.some(b => { return o.id === b.id }) && !toRemove.some(b => { return o.id === b.id }) && !(new Date(o.departure_time) < new Date())
                    )
                    const newArray = toAdd.concat(filtered);
                    console.log(filtered)
                    console.log(newArray)
                    filtered.forEach(c => {
                        addData(Stores.Arrival, c)
                    })

                    newArray.sort((a, b) => new Date(a.departure_time).getTime() - new Date(b.departure_time).getTime())
                    setZones(newArray);
                })
                .catch((error) => {
                    if (!!error.isAxiosError && !error.response) {
                        //
                    }
                });
        }, 5000)

        return () => clearInterval(intervalId);
    }, [origin, zones])

    return { zones, error };
};
